import * as SensorJSInterface from './SensorJSInterface';
import log from 'loglevel';

export default class SensorJSDriver {

    constructor(driverName) {

        this.driverName = driverName;
        this.listeners = [];
        this._listeners = {};

        this.addListener = (callback)=> {
            log.debug(`${driverName || ''}::adding app listener`);
            this.listeners.push(callback);
            return this;
        }

        this.removeListener = (callback)=> {
            log.debug(`${driverName || ''}::removing app listener`);
            const idx = this.listeners.indexOf(callback);
            if (idx>=0) this.listeners.splice(idx,1);
            return this;
        }
    }

    /**
     * Returns the interface methods implemented by the supported driver.
     * Not all are mandatory. See SensorJSInterface class for details.
     * 
     * This getter binds a class roperty to a function named as rtcInterfaceId value
     * that is generated by the SensorJSInterface class using Symbol function
     */
    get [SensorJSInterface.rtcInterfaceId]() {
        return {
            connect: this.connect,
            reconnect: this.reconnect,
            disconnect: this.disconnect,
            subscribe: this.subscribe,
            unsubscribe: this.unsubscribe,
            publish: this.publish,
            addListener: this.addListener,
            removeListener: this.removeListener,
            release: this.release,
            presence: this.presence,
            connected: this.connected,
            connection: this.connection
        }
    }
}