const _rtcInterfaceId = Symbol('rtcInterface');

export const rtcInterfaceId = _rtcInterfaceId;

// properties that MUST be implemented by the implementing class
const properties = [
    'connect:function', 
    'disconnect:function', 
    'publish:function',
    'subscribe:function', 
    'unsubscribe:function'
];

/**
 * Checks whether the class implements all required methods
 * 
 * @param {Class object to be checked } obj 
 */
export const check = (obj) => {
    
    // get interface from object
    const driver = obj[_rtcInterfaceId];

    // if not implemented HALT!
    if (!driver) throw 'Object MUST implement rtcInterface';
    
    // check for mandatory attributes
    const missing = properties.some((property) => {
        const a = property.split(':');
        return ( !driver.hasOwnProperty(a[0]) || typeof(driver[a[0]]) !== a[1] );
    });

    // if someone is missing, HALT!
    if (missing) throw `Object MUST implement interface properties'`;
} 
