import SensorJSDriver from '../SensorJSDriver';
import * as SensorJSInterface from '../SensorJSInterface';
import log from 'loglevel';

const PubNub = require('pubnub');

export default class Pubnub extends SensorJSDriver {

    constructor(options, callback) {
        super('pubnub');
        
        const self = this;

        let firstConnect = true;
        let channels = [];
        let pubnub;

        log.debug('pubnub::config:' + JSON.stringify(options));

        // mandatory methods
        this.connect = ()=> {

            pubnub = new PubNub(options);

            log.info('pubnub::connecting...');

            pubnub.addListener(this.pubnubListener = {
                status(statusEvent) {
                    if (statusEvent.category === "PNConnectedCategory") {
                        log.info('pubnub::service connected');
                        if (firstConnect) {
                            firstConnect = false;
                            callback?.(self);
                        }
                    } else if (statusEvent.error) {
                        log.error(`pubnub:: category: ${statusEvent.category} - status: ${statusEvent.errorData.status} - message: ${statusEvent.errorData.message}`);
                        if (firstConnect) {
                            firstConnect = false;
                            callback?.();
                        }
                    }
                },
                message(event) {
                    log.debug(`pubnub::received message: ${event.message}`);
                    const msg = JSON.parse(event.message);
                    self.listeners.forEach((listener)=>{
                        listener(event.message.status, msg);
                    });
                },
                presence(presenceEvent) {
                    // TBI
                },
                error(event) {
                    log.error(`pubnub::ERROR ${JSON.stringify(event)}`);
                    callback?.(self, true);
                },
            });
        }

        this.reconnect = ()=> {
            pubnub.reconnect();
        }

        this.disconnect = ()=> {
            pubnub.stop();
        }

        this.release = () =>  {
            log.debug('pubnub::release');
            pubnub.unsubscribe(channels);
            channels = [];
            this.listeners = [];
            pubnub.removeListener(this.pubnubListener);
            pubnub.stop();
        }

        this.subscribe = (_channels) => {
            pubnub.unsubscribe(channels);
            if ( _channels instanceof Array ) {
                channels = channels.concat(_channels);
            } else {
                channels.push(_channels);
            }
            pubnub.subscribe({channels});
            log.debug(`pubnub::subscribed to channels: ${JSON.stringify(channels)}`);
            return this;
        }

        this.unsubscribe = (_channels) => {
            if (_channels instanceof Array) {
                pubnub.unsubscribe({ channels: _channels });
                _channels.forEach((channel)=>{
                    channels.splice(channels.indexOf(channel),1)
                });
                log.debug(`pubnub::unsubscribed from channels: ${JSON.stringify(_channels)}`);
            } else {
                pubnub.unsubscribe({ channels: [ _channels ] });
                channels.splice(channels.indexOf(_channels),1);
                log.debug(`pubnub::unsubscribed from channel: ${_channels}`);
            }
            return this;
        }
        
        this.publish = (channel, message, callback) => {
            this.pubnub.publish({ channel, message, callback });
            return this;
        }

        this.presence = ()=> {}

        this.connected = ()=> {};

        this.connection = ()=> {};

        // check RTC interface implementation.
        SensorJSInterface.check(this);

        // start connection
        this.connect();

        return this;
    }

}